import 'package:json_annotation/json_annotation.dart';

// import "LedgerDef.dart" show HistoAction;
// import "ILedgerIntf.dart" show ILedgerSalesConso, ILedgerSalesPay;
// --------------------------------------------------------------------------------

part 'ILedgerTbl.g.dart';

/** Histo aka Big Book */
@JsonSerializable(explicitToJson: true)
class ILedgerTblSeq {
  ILedgerTblSeq({
    required this.DateHeure,
    this.NoSeq,
    this.Station,
    this.FiscalId,
    required this.Staff,
    required this.StaffName,
    this.Teacher,
    this.TeacherName,
    this.PLU,
    this.Nom,
    this.NomProd,
    this.NomAdd,
    this.Action,
    this.Actions,
    this.Qty,
    this.Base,
    this.TTC,
    this.Pay,
    this.TotAdd,
    required this.Details,
  });

  factory ILedgerTblSeq.fromJson(Map<String, dynamic>? json) => _$ILedgerTblSeqFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerTblSeqToJson(this);

  /** Pointage Timestamp as String(yyyy-mm-dd hh:nn:ss) or as firestore.TimeStamp or ... */

  /** Registration Timestamp */
  @JsonKey(defaultValue: '')
  final String DateHeure;

  /** Sequence Number identifying a Block details of a Sale/Benefit
   * This Define a BLOCK in the Visu
   */
  final int? NoSeq;
  final int? Station;
  final String? FiscalId;

  /** Staff Number */
  @JsonKey(defaultValue: 0)
  final int Staff;
  @JsonKey(defaultValue: '')
  final String StaffName;

  /** Staff Number */
  final int? Teacher;
  final String? TeacherName;
  final String? PLU;
  final String? Nom;
  final String? NomProd;
  final String? NomAdd;

  /** Histo Action Flag
   *   O : Ouverture
   *   > : ReOuv
   *   L : Lancer
   *   + : Suite
   *   S : Split
   *   s : Transft
   *   z : FullTicket
   *   F : Pro Forma
   *   A : Addition
   *   C : Clôture
   *   D : Article
   *   M : Menu
   *   1 : Au Menu
   *   X : Article Externe
   *   Z : Article Container
   *   e : Hotel Room
   *   m : Mémo
   *   G : Message Simple
   *   g : Message.Linked
   *   K : Message Plus
   *   ! : Message Minus
   *   Q : Message Multiplic.Quantite
   *   U : Message Multiplic.Prix
   *   a : Message Au Menu
   *   R : Discount
   *   ^ : Multiplic.Discount
   *   % : Member Discount
   *   V : RestoPass Discount
   *   b : Voucher Discount
   *   ? : Info
   *   - : Séparation Conso
   *   = : Séparation Pay
   *   f : PxCM
   *   x : Drapeau
   *   N : TimeDrapeau
   *   q : TelNrDrapeau
   *   , : Réservation
   *   t : Info Room
   *   # : Info Member
   *   B : Info FreeVoucher
   *   I : Invoice
   *   n : Delivry/Take-Away Info
   *   H : Paiement
   *   H : Paiement Machine
   *   H : Paiement Jeton
   *   P : Paiement Member Account
   *   Y : Bon KDO
   *   € : Débour
   *   { : Ouv.Service
   *   } : Clot.Svc
   *   | : Clot.Box
   *   ( : Staff In
   *   ) : Staff Out
   *   @ : Info Prestation Staff
   *   [ : Staff Clock IN
   *   ] : Staff Clock Out
   *   & : Info Blocage
   *   . : Barcontrol IN
   *   / : Barcontrol REJECTED
   *   $ : Barcontrol OUT
   *   T : Tiroir
   *   p : Rpt
   *   J : Ticket Spl.
   *   k : Addition Duplicata
   *   i : Log Invoice Msg
   *   j : Log RePrint InvTicket Msg
   */
  final String? Action;
  final List<String>? Actions;
  final num? Qty;
  final num? Base;
  final num? TTC;
  final num? Pay;
  final num? TotAdd;

  /* ILedgerSalesConso | ILedgerSalesPay */
  @JsonKey(defaultValue: [])
  final List<dynamic> Details;
}
// --------------------------------------------------------------------------------

/** Histo aka Big Book */
@JsonSerializable(explicitToJson: true)
class ILedgerTbl {
  ILedgerTbl({
    this.SvcNumZ,
    this.Hre,
    this.NumSales,
    this.GUID,
    required this.Tbl,
    this.TblDescr,
    this.Member,
    this.MemberBarcode,
    this.Proprio,
    this.ProprioName,
    this.Cvts,
    this.Actions,
    this.HistoFlags,
    this.Tariff,
    this.Base,
    this.TTC,
    this.Pay,
    this.ToPaid,
    required this.Seqs,
  });

  factory ILedgerTbl.fromJson(Map<String, dynamic>? json) => _$ILedgerTblFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$ILedgerTblToJson(this);

  /** Unique Identifier Number of a Cloture Service ( aka Z). */
  final int? SvcNumZ;

  /** Internal Number identifying a Block details of a Sale/Benefit */
  final int? Hre;

  /** Sales Number. A Sale can be composed of multiple <NoSeq> blocks */
  final num? NumSales;
  final String? GUID;
  @JsonKey(defaultValue: '')
  final String Tbl;
  final String? TblDescr;
  final String? Member;
  final String? MemberBarcode;
  final int? Proprio;
  final String? ProprioName;

  /** Number of Seats */
  final int? Cvts;
  final List<String>? Actions;

  /** Histo Flags
   * Value differt according to ActionFlag
   * #Conso     : Article Flags
   * #Pay       : ModePay Flags
   * #CashDrawer: 1:Manual(NoTable=Z) 0:Auto
   * #Ouverture,Cloture : Histo Flags (hfHasClient,hfCpteClient,...)
   *   Bit 00 : HasFDMInfo
   *   Bit 01 : WithEmport
   *   Bit 02 : OnlyEmport
   *   Bit 03 : HasMenu
   *   Bit 04 : HasConsos
   *   Bit 05 : WithNONFOOD
   *   Bit 06 : OnlyNONFOOD
   *   Bit 07 : HasPay
   *   Bit 08 : HasPayElect
   *   Bit 09 : Blanco
   *   Bit 10 : CpteCLI
   *   Bit 11 : RechargeCLI
   *   Bit 12 : HasHappySpTariff
   *   Bit 13 : HasClient
   *   Bit 14 : WithFOOD
   *   Bit 15 : OnlyFOOD
   *   Bit 16 : PseudoTable
   *   Bit 17 : OnlyResto
   *   Bit 18 : WithResto
   *   Bit 19 : TaxExempted
   *   Bit 20 : AddMultiple
   *   Bit 21 : ExternalPLU
   *   Bit 22 : NewBonValueur
   *   Bit 23 : UsedBonValueur
   *   Bit 24 : HasInvoice
   *   Bit 25 : HasNegatif
   *   Bit 26 : HasSplit
   *   Bit 27 : HasForceAZero
   *   Bit 28 : HasOffert
   *   Bit 29 : HasDiscount
   *   Bit 30 : HasPrixSaisi
   *   Bit 31 : HasPayPending
   *   Bit 32 : HasErrLine
   *   Bit 33 : HasPayJeton
   *   Bit 34 : ExternalBonValueur
   *   Bit 35 : HasHotelLnk
   *   Bit 36 : HasChargeRoom
   *   Bit 37 : HasPayMachine
   *   Bit 38 : HasTicketSimple
   *   Bit 40 : IsTraining
   *   Bit 41 : SuperStaff
   *   Bit 43 : HasAfterAdd
   *   Bit 45 : HasSplitIn
   *   Bit 46 : HasSplitOut
   *   Bit 47 : HasNegatifAfterAdd
   *   Bit 48 : WithAlcool
   *   Bit 49 : WithDeluxe
   *   Bit 50 : ContainerPLU
   *   Bit 51 : RptModifFlag1
   *   Bit 52 : RptModifFlag2
   *   Bit 53 : RptModifFlag3
   *   Bit 54 : RptModifFlag4
   *   Bit 55 : HappyTimeOn
   *   Bit 56 : HasHappyPrice
   *   Bit 57 : HasHappyQt
   *   Bit 58 : HappyTimeOff
   *   Bit 59 : IsDelivry
   *   Bit 60 : IsPickup
   *   Bit 62 : Clotured
   */
  final String? HistoFlags;
  final num? Tariff;
  final num? Base;
  final num? TTC;
  final num? Pay;
  final num? ToPaid;
  @JsonKey(defaultValue: [])
  final List<ILedgerTblSeq> Seqs;
}
