import 'package:json_annotation/json_annotation.dart';

import "IR5BtnProperties.dart" show IR5BtnProperties;

part 'IR5ModePay.g.dart';

// --------------------------------------------------------------------------------
enum PayTipsMode { NoAutoTips, Tips, Retour }
const Map<int, PayTipsMode> PayTipsModes = {
  0: PayTipsMode.NoAutoTips,
  1: PayTipsMode.Tips,
  2: PayTipsMode.Retour,
};
int PayTipsModeToJson(PayTipsMode f) {
  return PayTipsModes.entries.firstWhere((element) => element.value == f).key;
}

// --------------------------------------------------------------------------------
enum PayType { Specie, PaperCheck, KDOCheck, PendingTransfer, DebitCard, PaperNoRetour, Jeton, CashMachine, BankMachine, TicketMachine, MemberAccount, DiffPay, Tips, Retour, PLURemise, PLUMsg }
const Map<int, PayType> PayTypes = {
  0: PayType.Specie,
  1: PayType.PaperCheck,
  2: PayType.KDOCheck,
  3: PayType.PendingTransfer,
  4: PayType.DebitCard,
  5: PayType.PaperNoRetour,
  10: PayType.Jeton,
  20: PayType.CashMachine,
  30: PayType.BankMachine,
  31: PayType.TicketMachine,
  40: PayType.MemberAccount,
  50: PayType.DiffPay,
  51: PayType.Tips,
  52: PayType.Retour,
  90: PayType.PLURemise,
  91: PayType.PLUMsg,
};
int PayTypeToJson(PayType f) {
  return PayTypes.entries.firstWhere((element) => element.value == f).key;
}

// --------------------------------------------------------------------------------
enum PayLink { NOPAYLINK, BSYS_VIC, BSYS_CTEP, LOYALTEK, CCV_SERIAL, CCV_HTTP, PAYCONIQ, INGENICO, VIVAWALLET, ADYEN, RESTOPASS, IZIII, GLORY, CASHDRO, SASSO }
const Map<int, PayLink> PayLinks = {
  0: PayLink.NOPAYLINK,
  1: PayLink.BSYS_VIC,
  2: PayLink.BSYS_CTEP,
  3: PayLink.LOYALTEK,
  4: PayLink.CCV_SERIAL,
  5: PayLink.CCV_HTTP,
  6: PayLink.PAYCONIQ,
  7: PayLink.INGENICO,
  8: PayLink.VIVAWALLET,
  9: PayLink.ADYEN,
  11: PayLink.RESTOPASS,
  12: PayLink.IZIII,
  21: PayLink.GLORY,
  22: PayLink.CASHDRO,
  23: PayLink.SASSO,
};
int PayLinkToJson(PayLink f) {
  return PayLinks.entries.firstWhere((element) => element.value == f).key;
}
// --------------------------------------------------------------------------------

/** Modes Paiements */
@JsonSerializable()
class IR5PayBillet {
  IR5PayBillet({
    required this.PLU,
    required this.Nom,
    required this.BilletVal,
  });

  factory IR5PayBillet.fromJson(Map<String, dynamic>? json) => _$IR5PayBilletFromJson(json ?? {});

  Map<String, dynamic> toJson() => _$IR5PayBilletToJson(this);

  /** Article Unique Code (Price Look-Up) */
  @JsonKey(defaultValue: '')
  final String PLU;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: 0)
  final num BilletVal;
}
// --------------------------------------------------------------------------------

/** Modes Paiements */
@JsonSerializable()
class IR5PayToken {
  IR5PayToken({
    required this.PLU,
    required this.Nom,
    required this.TokenVal1,
    required this.TokenVal2,
  });

  factory IR5PayToken.fromJson(Map<String, dynamic>? json) => _$IR5PayTokenFromJson(json ?? {});

  Map<String, dynamic> toJson() => _$IR5PayTokenToJson(this);

  /** Article Unique Code (Price Look-Up) */
  @JsonKey(defaultValue: '')
  final String PLU;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: 0)
  final num TokenVal1;
  @JsonKey(defaultValue: 0)
  final num TokenVal2;
}
// --------------------------------------------------------------------------------

/** Modes Paiements */
@JsonSerializable(explicitToJson: true)
class IR5ModePayBtn {
  IR5ModePayBtn({
    required this.PLU,
    required this.Nom,
    required this.BilletVal,
    required this.BtnProp,
    required this.Onglet,
    required this.Top,
    required this.Left,
    required this.Height,
    required this.Width,
    required this.Visible,
  });

  factory IR5ModePayBtn.fromJson(Map<String, dynamic>? json) => _$IR5ModePayBtnFromJson(json ?? {});

  Map<String, dynamic> toJson() => _$IR5ModePayBtnToJson(this);

  /** Article Unique Code (Price Look-Up) */
  @JsonKey(defaultValue: '')
  final String PLU;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: 0)
  final num BilletVal;

  final IR5BtnProperties BtnProp;
  @JsonKey(defaultValue: 0)
  final int Onglet;
  @JsonKey(defaultValue: 0)
  final num Top;
  @JsonKey(defaultValue: 0)
  final num Left;
  @JsonKey(defaultValue: 0)
  final num Height;
  @JsonKey(defaultValue: 0)
  final num Width;
  @JsonKey(defaultValue: false)
  final bool Visible;
}
// --------------------------------------------------------------------------------

/** Modes Paiements */
@JsonSerializable(explicitToJson: true)
class IR5ModePay {
  IR5ModePay({
    required this.PLU,
    required this.Nom,
    required this.Description,
    required this.PayType,
    required this.PayLink,
    required this.PayLinkParams,
    required this.PayTipsMode,
    required this.Flags,
    required this.NomFr,
    required this.NomNl,
    required this.NomUk,
    required this.ExtRef,
    required this.Tags,
    required this.WebOrder,
    required this.Categorie,
    required this.Account,
    required this.Stations,
    required this.Devise,
    required this.Billets,
    required this.Tokens,
  });

  factory IR5ModePay.fromJson(Map<String, dynamic>? json) => _$IR5ModePayFromJson(json ?? {});

  Map<String, dynamic> toJson() => _$IR5ModePayToJson(this);

  /** Article Unique Code (Price Look-Up) */
  @JsonKey(defaultValue: '')
  final String PLU;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String Nom;

  /** Just a free Description */
  @JsonKey(defaultValue: '')
  final String Description;

  /** ModePay Type
   *   00 : Specie
   *   01 : PaperCheck
   *   02 : KDOCheck
   *   03 : PendingTransfer
   *   04 : DebitCard
   *   05 : PaperNoRetour
   *   10 : Jeton
   *   20 : CashMachine
   *   30 : BankMachine
   *   31 : TicketMachine
   *   40 : MemberAccount
   *   50 : DiffPay
   *   51 : Tips
   *   52 : Retour
   *   90 : PLURemise
   *   91 : PLUMsg
   */
  @JsonKey(defaultValue: 0)
  final int /* num | PayType */ PayType;

  /** ModeLink Type
   *   01 : BankSys VIC 1.07
   *   02 : Atos CTEP
   *   03 : LoyalTek TransHandler
   *   04 : CCV Serial
   *   05 : CCV Http
   *   06 : Payconiq
   *   07 : Ingenico
   *   08 : VivaWallet
   *   09 : Adyen Terminal
   *   11 : RESTOPASS
   *   12 : IZIII
   *   21 : GLORY
   *   22 : CASHDRO
   *   23 : SASSO
   */
  @JsonKey(defaultValue: 0)
  final int /* num | PayLink */ PayLink;
  @JsonKey(defaultValue: '')
  final String PayLinkParams;
  @JsonKey(defaultValue: 0)
  final int /* num | PayTipsMode */ PayTipsMode;

  /** Button Flags
   *   Bit 00 : payElectro
   *   Bit 01 : payCashDrawer
   *   Bit 02 : payNomASaissir
   *   Bit 03 : payNoTipsAuto
   *   Bit 04 : payNoNegatif
   *   Bit 05 : payMaxSaldo
   *   Bit 06 : payDontAskMontan
   *   Bit 09 : MachinePay
   *   Bit 18 : RechargeCLI
   *   Bit 20 : payRecharge
   *   Bit 24 : payNoRounding
   *   Bit 25 : payOnlyManager
   *   Bit 26 : payOnlyIffMember
   *   Bit 27 : payOnlyIffNoMember
   *   Bit 29 : ForeignDevise
   *   Bit 35 : IsTips
   *   Bit 36 : IsRetour
   *   Bit 38 : payNoLancerAuto
   *   Bit 39 : paySavePointage
   *   Bit 40 : payDoCumulAuPointage
   *   Bit 41 : payDoCumulAuLancer
   *   Bit 42 : payDirectCashDrawer
   *   Bit 50 : payCashTicketNeeded
   *   Bit 51 : payCashTicketExcluded
   *   Bit 52 : payCashTicketOnlyIfPayIn
   *   Bit 53 : payCashTicketOnlyIfPayOut
   *   Bit 58 : payTextAlignCenter
   *   Bit 59 : payTextAlignBottom
   *   Bit 60 : payRptModifFlag1
   *   Bit 61 : payRptModifFlag2
   *   Bit 62 : payRptModifFlag3
   *   Bit 63 : payRptModifFlag4
   */
  @JsonKey(defaultValue: 0)
  final int Flags;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String NomFr;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String NomNl;

  /** Just a Name */
  @JsonKey(defaultValue: '')
  final String NomUk;

  /** External Ref (used as primary key) */
  @JsonKey(defaultValue: '')
  final String ExtRef;

  /** Free Tags or Labels semicolumn separated */
  @JsonKey(defaultValue: '')
  final String Tags;

  /** Export to WebOrder Platforms (semicolumn separated list of Platforms)
   * Export ModePay if ModePay.WebOrder
   * Export Article if Article.WebOrder or Article.Categ.WebOrder or Article.Group.WebOrder
   */
  @JsonKey(defaultValue: '')
  final String WebOrder;

  /** ModePay Categ : Free Use (for Types [PLURemise,PLUMsg] store the PLU to use) */
  @JsonKey(defaultValue: '')
  final String Categorie;
  @JsonKey(defaultValue: '')
  final String Account;
  @JsonKey(defaultValue: '')
  final String Stations;
  @JsonKey(defaultValue: '')
  final String Devise;
  @JsonKey(defaultValue: [])
  final List<IR5PayBillet> Billets;
  @JsonKey(defaultValue: [])
  final List<IR5PayToken> Tokens;
}
