import 'package:json_annotation/json_annotation.dart';

import "IR5Member.dart" show IR5Member;
import 'IR5PointageConso.dart' show IR5PointageConso;
import 'IR5PointagePay.dart' show IR5PointagePay;

// --------------------------------------------------------------------------------
part 'IR5ResPointage.g.dart';

/** Histo aka Big Book */
@JsonSerializable(explicitToJson: true)
class IR5ResPointage {
  IR5ResPointage({
    required this.NoTbl,
    required this.SsTbl,
    required this.Nom,
    required this.TblType,
    required this.Cvts,
    this.Descr,
    this.TblMode,
    this.OUFTIState,
    this.DteOuv,
    this.DteClot,
    this.NoHre,
    this.NoSales,
    this.GUID,
    this.DetailsFmt,
    this.StationNum,
    this.StationFiscalId,
    this.ProprioNo,
    this.ProprioNom,
    this.MemberCard,
    this.Member,
    this.Tariff,
    this.TariffIn,
    this.TariffOut,
    this.Consos,
    this.Pays,
    this.Details,
    this.Total,
    this.Base,
    this.Tax,
    this.EnCaisse,
  });

  factory IR5ResPointage.fromJson(Map<String, dynamic>? json) => _$IR5ResPointageFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5ResPointageToJson(this);

  @JsonKey(defaultValue: '')
  final String NoTbl;
  @JsonKey(defaultValue: '')
  final String SsTbl;

  /** Free Table Name. ex: nom + prénom + gsm du client */
  @JsonKey(defaultValue: '')
  final String Nom;
  @JsonKey(defaultValue: 0)
  final int TblType;
  @JsonKey(defaultValue: 0)
  final int Cvts;

  /** MemberName|TblName|... Only in GetPointage Answer */
  final String? Descr;

  /** TblMode: 0:ReadOnly | 1:ReadWrite| 2:TrainingReadOnly | 3:TrainingReadWrite */
  final int? TblMode;
  final String? OUFTIState;
  final String? DteOuv;
  final String? DteClot;
  final int? NoHre;

  /** Sales Number. A Sale can be composed of multiple <NoSeq> blocks */
  final int? NoSales;
  final String? GUID;

  /** DetailsFmt: Details | Addition | AddBySsAdd | AddBySeq. Only in SetPointage Request */
  final String? DetailsFmt;
  final int? StationNum;

  /** Station FiscalId */
  final String? StationFiscalId;
  final int? ProprioNo;
  final String? ProprioNom;

  /** Member NumCard. numéro identifiant du personnel (alphanum) */
  final String? MemberCard;
  final IR5Member? Member;

  /** Tariff used / to use (TariffIn,TariffOut Only in GetPointage Answer) */
  final int? Tariff;
  final int? TariffIn;
  final int? TariffOut;
  final List<IR5PointageConso>? Consos;
  final List<IR5PointagePay>? Pays;
  final List<dynamic>? Details;

  /** Totals Only in GetPointage Answer) */
  final num? Total;

  /** Total Price without VAT
   * ActionFlag:
   * -----------
   * # FreeVoucherInfo : Total Value of New FreeVoucher Generated (PointsTaked*Etab.PointValue)
   */
  final num? Base;
  final num? Tax;
  final num? EnCaisse;
}
