import 'package:json_annotation/json_annotation.dart';

part 'IR5Tbl.g.dart';
// --------------------------------------------------------------------------------

/** Restaurant Tables */
@JsonSerializable()
class IR5Tbl {
  IR5Tbl({
    required this.Num,
    required this.Nom,
    required this.Station,
    required this.Division,
    required this.Description,
    required this.ObjectType,
    required this.GotoSalle,
    required this.TblType,
    required this.TariffIn,
    required this.TariffOut,
    required this.Flags,
    required this.AskCvts,
    required this.Cvts,
    required this.AutoAsk,
    required this.AutoMember,
    required this.Bmp,
    required this.Angle,
    required this.Shape,
    required this.Top,
    required this.Left,
    required this.Height,
    required this.Width,
    required this.Svc_Top,
    required this.Svc_Left,
    required this.Svc_Height,
    required this.Svc_Width,
  });

  factory IR5Tbl.fromJson(Map<String, dynamic>? json) => _$IR5TblFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5TblToJson(this);

  /** Table
   * Default NumTable for mode Pos (Direct Pointing)
   * - Station Number OR
   * - "V"+Staff Number OR
   * - "VG"+Staff Number OR
   * - "VS"+Station Number
   */
  @JsonKey(defaultValue: '')
  final String Num;

  /** Table or Salle Name
   * Displayed only if diff from NoTable
   */
  @JsonKey(defaultValue: '')
  final String Nom;

  /** Station Number */
  @JsonKey(defaultValue: 0)
  final int Station;

  /** Division Number */
  @JsonKey(defaultValue: 0)
  final int Division;

  /** Just a free Description */
  @JsonKey(defaultValue: '')
  final String Description;

  /** Default ObjectType = tblTrueTbl */

  /** Table ObjectTypes
   *
   * 10 - <<<Table>>>
   * 11 - Alias
   * 12 - Members
   * 21 - Image
   * 20 - Image + Texte
   * 40 - Porte
   * 41 - Porte + Texte
   * 43 - Salles List
   * 44 - All Salles List
   * 42 - MiniSalle
   * 45 - WebView
   * 50 - Total Service
   * 98 - <<<Missing POS Table>>>
   * 1 - <<<Table>>>
   * 97 - <<<Missing Member Table>>>
   * 99 - <<<Missing Table>>>
   */
  @JsonKey(defaultValue: 0)
  final int ObjectType;
  @JsonKey(defaultValue: '')
  final String GotoSalle;

  /** Default TblType = tblNormal */

  /** Image Types
   * 11 - Normal
   * 12 - Comptoir
   * 13 - V.Tbl Bar
   * 14 - Personnel
   * 15 - Traiteur
   * 16 - Taverne
   * 17 - Delivery
   * 18 - Pickup
   * 19 - Auto Inc
   * 20 - Type J
   * 91 - SplitAPart
   * 92 - EnCaisse
   */
  @JsonKey(defaultValue: 0)
  final int TblType;

  /** Tariff In. Can be a TblTariff(1,2,3,4) or any TariffColor(Id>999) */

  /** Tariff Color to use if IN(resto) */
  @JsonKey(defaultValue: 0)
  final int TariffIn;

  /** Tariff Out. Can be a TblTariff(1,2,3,4) or any TariffColor(Id>999) */

  /** Tariff Color to use if OUT(emport) */
  @JsonKey(defaultValue: 0)
  final int TariffOut;

  /** Table Flags
   *   Bit 00 : TblIsBlocked
   *   Bit 01 : TblOnlyManager
   *   Bit 06 : AskTblMember
   *   Bit 07 : AskTblDesc
   *   Bit 08 : DontAskTblDesc
   *   Bit 10 : NoSplitFrom
   *   Bit 11 : NoSplitTo
   *   Bit 12 : SplitToTarif
   *   Bit 19 : IsForcedIn
   *   Bit 20 : IsForcedOut
   *   Bit 21 : IsForcedHappyHour
   *   Bit 22 : IsForcedFz
   *   Bit 30 : TarifIN1
   *   Bit 31 : TarifIN2
   *   Bit 32 : TarifIN3
   *   Bit 33 : TarifIN4
   *   Bit 40 : TarifOUT1
   *   Bit 41 : TarifOUT2
   *   Bit 42 : TarifOUT3
   *   Bit 43 : TarifOUT4
   *   Bit 51 : RptModifFlag1
   *   Bit 52 : RptModifFlag2
   *   Bit 53 : RptModifFlag3
   *   Bit 54 : RptModifFlag4
   *   Bit 62 : SquareRectFlag
   *   Bit 63 : EllipseFlag
   */
  @JsonKey(defaultValue: 0)
  final int Flags;

  /** AutoMember,AutoAskn,... Params. Only for true Table */
  @JsonKey(defaultValue: false)
  final bool AskCvts;
  @JsonKey(defaultValue: 0)
  final int Cvts;

  /** AutoAsk = [Desc|Member|False] */
  @JsonKey(defaultValue: '')
  final String AutoAsk;

  /** Auto Member à l'ouverture */
  @JsonKey(defaultValue: '')
  final String AutoMember;

  /** Image Num */
  @JsonKey(defaultValue: 0)
  final int Bmp;
  @JsonKey(defaultValue: 0)
  final num Angle;
  @JsonKey(defaultValue: '')
  final String Shape;
  @JsonKey(defaultValue: 0)
  final num Top;
  @JsonKey(defaultValue: 0)
  final num Left;
  @JsonKey(defaultValue: 0)
  final num Height;
  @JsonKey(defaultValue: 0)
  final num Width;
  @JsonKey(defaultValue: 0)
  final num Svc_Top;
  @JsonKey(defaultValue: 0)
  final num Svc_Left;
  @JsonKey(defaultValue: 0)
  final num Svc_Height;
  @JsonKey(defaultValue: 0)
  final num Svc_Width;
}
